@@ -22,11 +22,12 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 #include "editor_pane.h"
+
 #include "../controls.h"
 #include "bitmap_button.h"
 #include "bitmap_knob.h"
 #include "bitmap_popup.h"
-#include "presets_menu.h"
+#include "editor_menus.h"
 
 //#define ENABLE_LAYOUT_EDIT 1
 
@@ -175,20 +176,26 @@ on_unrealize (GtkWidget *widget, gpointer user_data)
 
 #endif /////////////////////////////////////////////////////////////////////////
 
+static gboolean g_is_plugin;
+
 static void
 on_control_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-	if (event->type == GDK_BUTTON_PRESS && event->button == 3)
-	{
-		const size_t param_index = (size_t)data;
-		modal_midi_learn(param_index);
+	if (event->button != 3) {
+		return;
 	}
+
+	if (g_is_plugin) {
+		return;
+	}
+
+	gtk_menu_popup (GTK_MENU (controller_menu_new((int)(long)data)), NULL, NULL, NULL, NULL, 0, event->time);
 }
 
 static gboolean
-button_press_event (GtkWidget *widget, GdkEventButton *event, GtkWidget *presets_menu)
+button_release_event (GtkWidget *widget, GdkEventButton *event, GtkWidget *presets_menu)
 {
-	if (!(event->type == GDK_BUTTON_PRESS && event->button == 3)) {
+	if (event->button != 3) {
 		return FALSE;
 	}
 	gtk_menu_popup (GTK_MENU (presets_menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time ());
@@ -215,7 +222,7 @@ button_press_event (GtkWidget *widget, GdkEventButton *event, GtkWidget *presets
 #define KEY_CONTROL_PARAM_NUM	"param_num"
 
 GtkWidget *
-editor_pane_new (GtkAdjustment **adjustments, gboolean enable_popup_menu)
+editor_pane_new (void *synthesizer, GtkAdjustment **adjustments, gboolean is_plugin)
 {
 	static int initialised;
 	if (!initialised) {
@@ -234,6 +241,8 @@ editor_pane_new (GtkAdjustment **adjustments, gboolean enable_popup_menu)
 		);
 	}
 
+	g_is_plugin = is_plugin;
+
 	GtkWidget *fixed = gtk_fixed_new ();
 	gtk_widget_set_usize (fixed, 400, 300);
 	
@@ -402,10 +411,10 @@ editor_pane_new (GtkAdjustment **adjustments, gboolean enable_popup_menu)
 
 	GtkWidget *eventbox = gtk_event_box_new ();
 	gtk_container_add (GTK_CONTAINER (eventbox), fixed);
-	if (enable_popup_menu) {
-		GtkWidget *presets_menu = presets_menu_new (adjustments);
-		gtk_menu_attach_to_widget (GTK_MENU (presets_menu), eventbox, NULL);
-		g_signal_connect (eventbox, "button-press-event", G_CALLBACK (button_press_event), presets_menu);
+	if (is_plugin) {
+		GtkWidget *menu = editor_menu_new (synthesizer, adjustments);
+		gtk_menu_attach_to_widget (GTK_MENU (menu), eventbox, NULL);
+		g_signal_connect (eventbox, "button-release-event", G_CALLBACK (button_release_event), menu);
 	}
 
 	return eventbox;
