@@ -22,15 +22,17 @@
 #ifndef _editor_pane_h
 #define _editor_pane_h
 
+#include "../controls.h"
+
 #include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
-GtkWidget * editor_pane_new (GtkAdjustment **adjustments, gboolean enable_popup_menu);
+GtkWidget *
+editor_pane_new(void *synthesizer, GtkAdjustment **adjustments, gboolean is_plugin);
 
-void modal_midi_learn(int param_index);
+void modal_midi_learn(Param param_index);
 
 G_END_DECLS
 
 #endif
-
