@@ -1,7 +1,7 @@
 /*
  *  JackOutput.cc
  *
- *  Copyright (c) 2001-2015 Nick Dowell
+ *  Copyright (c) 2001-2016 Nick Dowell
  *
  *  This file is part of amsynth.
  *
@@ -22,7 +22,7 @@
 #include "JackOutput.h"
 
 #include "Configuration.h"
-#include "VoiceAllocationUnit.h"
+#include "midi.h"
 
 #if HAVE_JACK_MIDIPORT_H
 #include <jack/midiport.h>
@@ -32,11 +32,10 @@
 #include <jack/session.h>
 #endif
 
-#include <assert.h>
 #include <iostream>
-#include <string.h>
+#include <sstream>
 #include <stdio.h>
-#include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 #include <vector>
 
@@ -56,6 +55,7 @@ JackOutput::JackOutput()
 ,	l_port(NULL)
 ,	r_port(NULL)
 ,	m_port(NULL)
+,	m_port_out(NULL)
 ,	client(NULL)
 #endif
 {
@@ -97,6 +97,7 @@ JackOutput::init()
 #if HAVE_JACK_MIDIPORT_H
 	/* create midi input port(s) */
 	m_port = jack_port_register(client, "midi_in", JACK_DEFAULT_MIDI_TYPE, JackPortIsInput, 0);
+	m_port_out = jack_port_register(client, "midi_out", JACK_DEFAULT_MIDI_TYPE, JackPortIsOutput, 0);
 #endif
 	
 #if HAVE_JACK_SESSION_H
@@ -104,6 +105,9 @@ JackOutput::init()
 #endif
 
 	config.current_audio_driver = "JACK";
+	if (m_port) {
+		config.current_midi_driver = "JACK";
+	}
 	config.sample_rate = jack_get_sample_rate(client);
 	config.buffer_size = jack_get_buffer_size(client);
 	config.jack_client_name = std::string(jack_get_client_name(client));
@@ -150,10 +154,19 @@ JackOutput::process (jack_nframes_t nframes, void *arg)
 		}
 	}
 #endif
-	if (self->mAudioCallback != NULL) {
-		(*self->mAudioCallback)(lout, rout, nframes, 1,
-			midi_events.empty() ? NULL : &midi_events[0], midi_events.size());
+	std::vector<amsynth_midi_cc_t> midi_out;
+	amsynth_audio_callback(lout, rout, nframes, 1, midi_events, midi_out);
+#if HAVE_JACK_MIDIPORT_H
+	if (self->m_port_out) {
+		void *port_buffer = jack_port_get_buffer(self->m_port_out, nframes);
+		jack_midi_clear_buffer(port_buffer);
+		std::vector<amsynth_midi_cc_t>::const_iterator out_it;
+		for (out_it = midi_out.begin(); out_it != midi_out.end(); ++out_it) {
+			jack_midi_data_t data[] = { MIDI_STATUS_CONTROLLER, out_it->cc, out_it->value};
+			jack_midi_event_write(port_buffer, 0, data, 3);
+		}
 	}
+#endif
 	return 0;
 }
 #endif
