@@ -1,7 +1,7 @@
 /*
  *  MidiController.cc
  *
- *  Copyright (c) 2001-2012 Nick Dowell
+ *  Copyright (c) 2001-2016 Nick Dowell
  *
  *  This file is part of amsynth.
  *
@@ -22,8 +22,8 @@
 #include "MidiController.h"
 
 #include "Configuration.h"
-#include "drivers/MidiDriver.h"
 #include "midi.h"
+#include "VoiceBoard/Synth--.h"
 
 #include <assert.h>
 #include <cmath>
@@ -37,8 +37,6 @@ MidiController::MidiController()
 ,	_handler(NULL)
 ,	_rpn_msb(0xff)
 ,	_rpn_lsb(0xff)
-,	_config_needs_save(false)
-,	_midiDriver(NULL)
 {
 	presetController = 0;
 	Configuration & config = Configuration::get();
@@ -46,12 +44,6 @@ MidiController::MidiController()
 	loadControllerMap();
 }
 
-MidiController::~MidiController()
-{
-	if (_config_needs_save)
-		saveControllerMap();
-}
-
 void
 MidiController::HandleMidiData(const unsigned char* bytes, unsigned numBytes)
 {
@@ -71,7 +63,7 @@ MidiController::HandleMidiData(const unsigned char* bytes, unsigned numBytes)
 		}
 		// now we have at least one data byte
 
-		if (config.midi_channel && ((int) channel != config.midi_channel-1)) break;
+		bool ignore = config.midi_channel && ((int) channel != config.midi_channel - 1);
 
 		switch (status & 0xf0)
 		{
@@ -82,7 +74,7 @@ MidiController::HandleMidiData(const unsigned char* bytes, unsigned numBytes)
 				data = byte;
 				break;
 			}
-			dispatch_note( channel, data, 0 );
+			if (!ignore) dispatch_note( channel, data, 0 );
 			data = 0xff;
 			break;
 	
@@ -91,7 +83,7 @@ MidiController::HandleMidiData(const unsigned char* bytes, unsigned numBytes)
 				data = byte;
 				break;
 			}
-			dispatch_note(channel, data, byte);
+				if (!ignore) dispatch_note(channel, data, byte);
 			data = 0xff;
 			break;
 	
@@ -108,12 +100,12 @@ MidiController::HandleMidiData(const unsigned char* bytes, unsigned numBytes)
 				data = byte;
 				break;
 			}
-			controller_change(data, byte);
+				if (!ignore) controller_change(data, byte);
 			data = 0xFF;
 			break;
 
 		case MIDI_STATUS_PROGRAM_CHANGE:
-			if (presetController->getCurrPresetNumber() != byte) {
+			if (!ignore && presetController->getCurrPresetNumber() != byte) {
 				if (_handler) _handler->HandleMidiAllSoundOff();
 				presetController->selectPreset((int) byte);
 			}
@@ -133,7 +125,7 @@ MidiController::HandleMidiData(const unsigned char* bytes, unsigned numBytes)
 			}
 			int bend; bend = (int) ((data & 0x7F) | ((byte & 0x7F) << 7));
 			float fbend; fbend = (float) (bend - 0x2000) / (float) (0x2000);
-			pitch_wheel_change(fbend);
+			if (!ignore) pitch_wheel_change(fbend);
 			data = 0xFF;
 			break;
 	
@@ -166,11 +158,26 @@ MidiController::dispatch_note(unsigned char, unsigned char note, unsigned char v
 void
 MidiController::controller_change(unsigned char cc, unsigned char value)
 {
+	_midi_cc_vals[cc] = value;
+
+	if (last_active_controller.getValue() != cc)
+		last_active_controller.setValue(cc);
+
 	if (!_handler || !presetController)
 		return;
 
+	int paramId = _cc_to_param_map[cc];
+	if (paramId >= 0) {
+		presetController->getCurrentPreset().getParameter(paramId).SetNormalisedValue(value / 127.0f);
+		return; // MIDI CCs mapped by the user take precedence over default behaviour
+	}
+
 	switch (cc) {
-		case MIDI_CC_BANK_SELECT_LSB:
+		case MIDI_CC_BANK_SELECT_LSB: {
+			presetController->selectBank(value);
+			presetController->selectPreset(presetController->getCurrPresetNumber());
+			break;
+		}
 		case MIDI_CC_BANK_SELECT_MSB:
 			break;
 		case MIDI_CC_PAN_MSB: {
@@ -219,22 +226,8 @@ MidiController::controller_change(unsigned char cc, unsigned char value)
 			_handler->HandleMidiAllNotesOff();
 		case MIDI_CC_MODULATION_WHEEL:
 		default:
-			if (last_active_controller.getValue() != cc)
-				last_active_controller.setValue(cc);
-			int paramId = _cc_to_param_map[cc];
-			if (paramId >= 0)
-				presetController->getCurrentPreset().getParameter(paramId).SetNormalisedValue(value / 127.0f);
-			_midi_cc_vals[cc] = value;
 			break;
 	}
-	return;
-}
-
-void
-MidiController::timer_callback()
-{
-	if (_config_needs_save)
-		saveControllerMap();
 }
 
 void
@@ -252,8 +245,6 @@ MidiController::clearControllerMap()
 	_param_to_cc_map[kAmsynthParameter_LFOToOscillators] = 1;
 	_cc_to_param_map[7] = kAmsynthParameter_MasterVolume;
 	_param_to_cc_map[kAmsynthParameter_MasterVolume] = 7;
-
-	_config_needs_save = false;
 }
 
 void
@@ -261,6 +252,10 @@ MidiController::loadControllerMap()
 {
 	clearControllerMap();
 
+#if _WIN32
+	return;
+#endif
+
 	std::string fname(getenv("HOME"));
 	fname += "/.amSynthControllersrc";
 	std::ifstream file(fname.c_str(), std::ios::out);
@@ -272,13 +267,14 @@ MidiController::loadControllerMap()
 		_param_to_cc_map[paramId] = cc;
 	}
 	file.close();
-
-	_config_needs_save = false;
 }
 
 void
 MidiController::saveControllerMap()
 {
+#if _WIN32
+	return;
+#endif
 	std::string fname(getenv("HOME"));
 	fname += "/.amSynthControllersrc";
 	std::ofstream file(fname.c_str(), std::ios::out);
@@ -290,19 +286,17 @@ MidiController::saveControllerMap()
 		file << (name ? name : "null") << std::endl;
 	}
 	file.close();
-
-	_config_needs_save = false;
 }
 
 int
-MidiController::getControllerForParameter(int paramId)
+MidiController::getControllerForParameter(Param paramId)
 {
 	assert(0 <= paramId && paramId < kAmsynthParameterCount);
 	return _param_to_cc_map[paramId];
 }
 
 void
-MidiController::setControllerForParameter(int paramId, int cc)
+MidiController::setControllerForParameter(Param paramId, int cc)
 {
 	assert(paramId < kAmsynthParameterCount && cc < MAX_CC);
 
@@ -320,7 +314,7 @@ MidiController::setControllerForParameter(int paramId, int cc)
 		_cc_to_param_map[cc] = paramId;
 	}
 
-	_config_needs_save = true;
+	saveControllerMap();
 }
 
 void
@@ -331,26 +325,18 @@ MidiController::set_midi_channel	( int ch )
 	config.midi_channel = ch;
 }
 
-int
-MidiController::sendMidi_values       ()
-{
-	send_changes(true);
-	return 0;
-}
-
 void
-MidiController::send_changes(bool force)
+MidiController::generateMidiOutput(std::vector<amsynth_midi_cc_t> &output)
 {
-	if (!_midiDriver)
-		return;
-	for (size_t paramId = 0; paramId < kAmsynthParameterCount; paramId++) {
+	for (int paramId = 0; paramId < kAmsynthParameterCount; paramId++) {
 		int cc = _param_to_cc_map[paramId];
 		if (0 <= cc && cc < MAX_CC) {
 			Parameter &parameter = presetController->getCurrentPreset().getParameter(paramId);
-			unsigned char value = parameter.GetNormalisedValue() * 127.0;
-			if (_midi_cc_vals[cc] != value || force) {
+			unsigned char value = (unsigned char) roundf(parameter.GetNormalisedValue() * 127.0f);
+			if (_midi_cc_vals[cc] != value) {
 				_midi_cc_vals[cc] = value;
-				_midiDriver->write_cc(channel, cc, value);
+				amsynth_midi_cc_t out = { channel, (unsigned char)cc, value };
+				output.push_back(out);
 			}
 		}
 	}
