@@ -1,7 +1,7 @@
 /*
  *  Preset.cc
  *
- *  Copyright (c) 2001-2014 Nick Dowell
+ *  Copyright (c) 2001-2017 Nick Dowell
  *
  *  This file is part of amsynth.
  *
@@ -21,12 +21,21 @@
 
 #include "Preset.h"
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "controls.h"
 #include "VoiceBoard/LowPassFilter.h"
 
+#include <cassert>
 #include <cstdlib>
 #include <cstdio>
 #include <map>
+#include <iterator>
+
+#include "gettext.h"
+#define _(string) gettext (string)
 
 #ifdef _DEBUG
 #include <iostream>
@@ -39,34 +48,6 @@ Parameter TimeParameter (const std::string name, Param id)
 	return Parameter (name, id, 0, 0, 2.5f, 0, Parameter::PARAM_POWER, 3, 0.0005f, "s");
 }
 
-const char *osc_waveform_names[] = {
-	"sine", "square / pulse", "saw / triangle", "white noise", "noise + sample & hold", NULL
-};
-
-const char *lfo_waveform_names[] = {
-	"sine", "square", "triangle", "noise", "noise + sample & hold", "sawtooth (up)", "sawtooth (down)", NULL
-};
-
-const char *keyboard_mode_names[] = {
-	"poly", "mono", "legato", NULL
-};
-
-const char *filter_type_names[] = {
-	"low pass", "high pass", "band pass", "notch", "bypass", NULL
-};
-
-const char *filter_slope_names[] = {
-	"12 dB / octave", "24 dB / octave", NULL
-};
-
-const char *freq_mod_osc_names[] = {
-	"osc 1+2", "osc 1", "osc 2", NULL
-};
-
-const char *portamento_mode_names[] = {
-	"always", "legato", NULL
-};
-
 Preset::Preset			(const std::string name)
 :	mName (name)
 ,	nullparam ("null", kAmsynthParameterCount)
@@ -113,21 +94,14 @@ Preset::Preset			(const std::string name)
 	mParameters.push_back (Parameter		("filter_vel_sens",		kAmsynthParameter_FilterKeyVelocityAmount, 1));
 	mParameters.push_back (Parameter		("amp_vel_sens",		kAmsynthParameter_AmpVelocityAmount, 1));
 	mParameters.push_back (Parameter		("portamento_mode",		kAmsynthParameter_PortamentoMode, PortamentoModeAlways));
-
-	getParameter(kAmsynthParameter_Oscillator1Waveform).setValueStrings(osc_waveform_names);
-	getParameter(kAmsynthParameter_Oscillator2Waveform).setValueStrings(osc_waveform_names);
-	getParameter(kAmsynthParameter_LFOWaveform).setValueStrings(lfo_waveform_names);
-	getParameter(kAmsynthParameter_KeyboardMode).setValueStrings(keyboard_mode_names);
-	getParameter(kAmsynthParameter_FilterType).setValueStrings(filter_type_names);
-	getParameter(kAmsynthParameter_FilterSlope).setValueStrings(filter_slope_names);
-	getParameter(kAmsynthParameter_LFOOscillatorSelect).setValueStrings(freq_mod_osc_names);
-	getParameter(kAmsynthParameter_PortamentoMode).setValueStrings(portamento_mode_names);
 }
 
 Preset&
 Preset::operator =		(const Preset &rhs)
 {
     for (unsigned i=0; i<rhs.ParameterCount(); i++) {
+		if (shouldIgnoreParameter(i))
+			continue;
 		getParameter(i).setValue(rhs.getParameter(i).getValue());
     }
     setName(rhs.getName());
@@ -138,6 +112,8 @@ bool
 Preset::isEqual(const Preset &rhs)
 {
 	for (unsigned i = 0; i < mParameters.size(); i++) {
+		if (shouldIgnoreParameter(i))
+			continue;
 		if (getParameter(i).getValue() != rhs.getParameter(i).getValue()) {
 			return false;
 		}
@@ -175,20 +151,18 @@ Preset::AddListenerToAll	(UpdateListener* ul)
 	for (unsigned i=0; i<mParameters.size(); i++) getParameter(i).addUpdateListener (*ul);
 }
 
-std::string
-Preset::toString()
+void
+Preset::toString(std::stringstream &stream)
 {
-	std::stringstream stream;
 	stream << "amSynth1.0preset" << std::endl;
 	stream << "<preset> " << "<name> " << getName() << std::endl;
 	for (unsigned n = 0; n < ParameterCount(); n++) {
 		stream << "<parameter> " << getParameter(n).getName() << " " << getParameter(n).getValue() << std::endl;
 	}
-	return stream.str();
 }
 
 bool
-Preset::fromString(std::string str)
+Preset::fromString(const std::string &str)
 {
 	std::stringstream stream (str);
 
@@ -322,15 +296,138 @@ int parameter_get_display (int parameter_index, float parameter_value, char *buf
 		case kAmsynthParameter_FilterKeyVelocityAmount:
 		case kAmsynthParameter_AmpVelocityAmount:
 			return snprintf(buffer, maxlen, "%d %%", (int)roundf(parameter.GetNormalisedValue() * 100.0));
-		case kAmsynthParameter_FilterType:
-			return snprintf(buffer, maxlen, "%s", filter_type_names[(int)real_value]);
+		case kAmsynthParameter_FilterType: {
+            const char **filter_type_names = parameter_get_value_strings(parameter_index);
+            if (filter_type_names) {
+                return snprintf(buffer, maxlen, "%s", filter_type_names[(int)real_value]);
+            } else {
+                return snprintf(buffer, maxlen, "");
+            }
+        }
 	}
 	return 0;
 }
 
 const char **parameter_get_value_strings (int parameter_index)
 {
-	Parameter parameter = _preset.getParameter(parameter_index);
-	const char **value_strings = parameter.valueStrings();
-	return value_strings;
+    static const char **parameterStrings[kAmsynthParameterCount];
+    if (parameter_index < 0 || parameter_index >= kAmsynthParameterCount)
+        return NULL;
+
+    const char **strings = parameterStrings[parameter_index];
+    if (!strings) {
+        size_t i = 0, size = 0;
+        switch (parameter_index) {
+            case kAmsynthParameter_Oscillator1Waveform:
+            case kAmsynthParameter_Oscillator2Waveform:
+                strings = (const char **)calloc((size = 6), sizeof(char *));
+                strings[i++] = _("sine");
+                strings[i++] = _("square / pulse");
+                strings[i++] = _("triangle / saw");
+                strings[i++] = _("white noise");
+                strings[i++] = _("noise + sample & hold");
+                assert(i < size);
+                break;
+
+            case kAmsynthParameter_LFOWaveform:
+                strings = (const char **)calloc((size = 8), sizeof(char *));
+                strings[i++] = _("sine");
+                strings[i++] = _("square");
+                strings[i++] = _("triangle");
+                strings[i++] = _("noise");
+                strings[i++] = _("noise + sample & hold");
+                strings[i++] = _("sawtooth (up)");
+                strings[i++] = _("sawtooth (down)");
+                assert(i < size);
+                break;
+
+            case kAmsynthParameter_KeyboardMode:
+                strings = (const char **)calloc((size = 4), sizeof(char *));
+                strings[i++] = _("poly");
+                strings[i++] = _("mono");
+                strings[i++] = _("legato");
+                assert(i < size);
+                break;
+
+            case kAmsynthParameter_FilterType:
+                strings = (const char **)calloc((size = 6), sizeof(char *));
+                strings[i++] = _("low pass");
+                strings[i++] = _("high pass");
+                strings[i++] = _("band pass");
+                strings[i++] = _("notch");
+                strings[i++] = _("bypass");
+                assert(i < size);
+                break;
+
+            case kAmsynthParameter_FilterSlope:
+                strings = (const char **)calloc((size = 3), sizeof(char *));
+                strings[i++] = _("12 dB / octave");
+                strings[i++] = _("24 dB / octave");
+                assert(i < size);
+                break;
+
+            case kAmsynthParameter_LFOOscillatorSelect:
+                strings = (const char **)calloc((size = 4), sizeof(char *));
+                strings[i++] = _("osc 1+2");
+                strings[i++] = _("osc 1");
+                strings[i++] = _("osc 2");
+                assert(i < size);
+                break;
+
+            case kAmsynthParameter_PortamentoMode:
+                strings = (const char **)calloc((size = 3), sizeof(char *));
+                strings[i++] = _("always");
+                strings[i++] = _("legato");
+                assert(i < size);
+                break;
+
+            default:
+                break;
+        }
+        parameterStrings[parameter_index] = strings;
+    }
+
+    return strings;
+}
+
+static bool s_ignoreParameter[kAmsynthParameterCount];
+
+bool Preset::shouldIgnoreParameter(int parameter)
+{
+	return s_ignoreParameter[parameter];
+}
+
+void Preset::setShouldIgnoreParameter(int parameter, bool ignore)
+{
+	s_ignoreParameter[parameter] = ignore;
+}
+
+std::string Preset::getIgnoredParameterNames()
+{
+	std::string names;
+	for (int i = 0; i < kAmsynthParameterCount; i++) {
+		if (shouldIgnoreParameter(i)) {
+			if (!names.empty())
+				names += " ";
+			names += _preset.getParameter(i).getName();
+		}
+	}
+	return names;
+}
+
+void Preset::setIgnoredParameterNames(std::string names)
+{
+	for (int i = 0; i < kAmsynthParameterCount; i++) {
+		setShouldIgnoreParameter(i, false);
+	}
+
+	std::stringstream ss(names);
+	std::istream_iterator<std::string> begin(ss);
+	std::istream_iterator<std::string> end;
+	std::vector<std::string> vstrings(begin, end);
+
+	std::vector<std::string>::const_iterator name_it;
+	for (name_it = vstrings.begin(); name_it != vstrings.end(); ++name_it) {
+		setShouldIgnoreParameter(parameter_index_from_name(name_it->c_str()), true);
+	}
 }
