@@ -26,13 +26,18 @@
 #include <cassert>
 #include <cstdlib>
 #include <cstring>
-#include <dirent.h>
 #include <string>
 #include <fstream>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
 
+#if !_WIN32
+#include <dirent.h>
+#include <unistd.h>
+#endif
+
+#include "gettext.h"
+#define _(string) gettext (string)
 
 using namespace std;
 
@@ -41,15 +46,14 @@ PresetController::PresetController	()
 :	bank_file ("")
 ,	updateListener (0)
 ,	nullpreset ("null preset")
+,	currentBankNo (-1)
 ,	currentPresetNo (-1)
 ,	lastPresetsFileModifiedTime (0)
 {
-	presets = new Preset [kNumPresets];
 }
 
 PresetController::~PresetController	()
 {
-	delete[] presets;
 	clearChangeBuffers();
 }
 
@@ -64,13 +68,6 @@ PresetController::selectPreset		(const int presetNo)
 	return 0;
 }
 
-int 
-PresetController::selectPreset		(const string name)
-{
-	for (int i=0; i<kNumPresets; i++) if (getPreset(i).getName() == name) return selectPreset (i);
-	return -1;
-}
-
 bool
 PresetController::containsPresetWithName(const string name)
 {
@@ -80,25 +77,13 @@ PresetController::containsPresetWithName(const string name)
 	return false;
 }
 
-Preset&
-PresetController::getPreset			(const string name)
-{
-	for (int i=0; i<kNumPresets; i++) if (getPreset(i).getName() == name) return getPreset (i);
-	return nullpreset;
-}
-
-int
-PresetController::newPreset			()
-{
-	for (int i=0; i<kNumPresets; i++) if (getPreset(i).getName() == "New Preset") return selectPreset (i);
-	return -1;
-}
-
 void
-PresetController::deletePreset		()
+PresetController::clearPreset		()
 {
+	loadPresets();
 	currentPreset = blankPreset;
-	notify ();
+	commitPreset();
+	savePresets();
 	clearChangeBuffers();
 }
 
@@ -202,7 +187,7 @@ PresetController::importPreset		(const string filename)
 	}
 }
 
-static unsigned long mtime(const char *filename)
+static long int mtime(const char *filename)
 {
 	struct stat st;
 	if (stat(filename, &st) != 0) {
@@ -260,8 +245,10 @@ static bool is_amsynth_file(const char *filename)
 	if (stat(filename, &st) < 0)
 		return false;
 
+#if !_WIN32
 	if (!S_ISREG(st.st_mode))
 		return false;
+#endif
 
 	FILE *file = fopen(filename, "r");
 	if (!file)
@@ -284,7 +271,11 @@ static off_t file_read_contents(const char *filename, void **result)
 	if (!file)
 		return 0;
 	fseek(file, 0, SEEK_END);
+#if _WIN32
+	long length = ftell(file);
+#else
 	off_t length = ftello(file);
+#endif
 	void *buffer = calloc(length + 1, 1);
 	fseek(file, 0, SEEK_SET);
 	fread(buffer, length, 1, file);
@@ -316,22 +307,17 @@ static float float_from_string(const char *s)
 	return rez * fact;
 }
 
-int
-PresetController::loadPresets		(const char *filename)
+static bool readBankFile(const char *filename, Preset *presets)
 {
-	if (filename == NULL)
-		filename = bank_file.c_str();
-
-	if (!is_amsynth_file(filename))
-		return -1;
-
-	if (strcmp(filename, bank_file.c_str()) == 0 && lastPresetsFileModifiedTime == mtime(filename))
-		return 0; // file not modified since last load
-
 	void *buffer = NULL;
 	off_t buffer_length = file_read_contents(filename, &buffer);
 	if (!buffer)
-		return -1;
+		return false;
+
+	if (memcmp(buffer, amsynth_file_header, sizeof(amsynth_file_header)) != 0) {
+		free(buffer);
+		return false;
+	}
 
 	char *buffer_end = ((char *)buffer) + buffer_length;
 
@@ -346,7 +332,7 @@ PresetController::loadPresets		(const char *filename)
 			if (strncmp(line_ptr, preset_prefix, sizeof(preset_prefix) - 1) == 0) {
 				presets[++preset_index] = Preset(std::string(line_ptr + sizeof(preset_prefix) - 1));
 			}
-			
+
 			static char parameter_prefix[] = "<parameter> ";
 			if (strncmp(line_ptr, parameter_prefix, sizeof(parameter_prefix) - 1) == 0) {
 				char *ptr = line_ptr + sizeof(parameter_prefix) - 1;
@@ -362,15 +348,60 @@ PresetController::loadPresets		(const char *filename)
 			line_ptr = end_ptr;
 		}
 	}
-	for (preset_index++; preset_index < kNumPresets; preset_index++)
+	for (preset_index++; preset_index < PresetController::kNumPresets; preset_index++)
 		presets[preset_index] = Preset();
 	free(buffer);
 
+	return true;
+}
+
+int
+PresetController::loadPresets		(const char *filename)
+{
+	if (filename == NULL)
+		filename = bank_file.c_str();
+
+	long int fileModifiedTime = mtime(filename);
+	if (strcmp(filename, bank_file.c_str()) == 0 && lastPresetsFileModifiedTime == fileModifiedTime)
+		return 0; // file not modified since last load
+
+	if (!readBankFile(filename, presets))
+		return -1;
+
+	currentBankNo = -1;
+	const std::vector<BankInfo> &banks = getPresetBanks();
+	for (int i = 0; i < banks.size(); i++) {
+		if (banks[i].file_path == (std::string) filename) {
+			currentBankNo = i;
+			break;
+		}
+	}
+
+	lastPresetsFileModifiedTime = fileModifiedTime;
 	bank_file = std::string(filename);
 
 	return 0;
 }
 
+void
+PresetController::selectBank(int bankNumber)
+{
+	const std::vector<BankInfo> &banks = getPresetBanks();
+
+	if (bankNumber >= banks.size())
+		return;
+
+	if (currentBankNo == bankNumber)
+		return;
+
+	for (int i = 0; i < kNumPresets; i++)
+		presets[i] = banks[bankNumber].presets[i];
+
+	currentBankNo = bankNumber;
+	bank_file = banks[bankNumber].file_path;
+	lastPresetsFileModifiedTime = mtime(banks[bankNumber].file_path.c_str());
+}
+
 ///////////////////////////////////
 
 static std::vector<BankInfo> s_banks;
@@ -381,7 +412,7 @@ static void scan_preset_bank(const std::string dir_path, const std::string file_
 
 	std::string bank_name = std::string(file_name);
 	if (bank_name == std::string(".amSynth.presets")) {
-		bank_name = "User bank";
+		bank_name = _("User bank");
 	} else {
 		std::string::size_type pos = bank_name.find_first_of(".");
 		if (pos != std::string::npos)
@@ -397,11 +428,13 @@ static void scan_preset_bank(const std::string dir_path, const std::string file_
 	bank_info.name = bank_name;
 	bank_info.file_path = file_path;
 	bank_info.read_only = read_only;
+	readBankFile(file_path.c_str(), bank_info.presets);
 	s_banks.push_back(bank_info);
 }
 
 static void scan_preset_banks(const std::string dir_path, bool read_only)
 {
+#if !_WIN32
 	DIR *dir = opendir(dir_path.c_str());
 	if (!dir)
 		return;
@@ -421,6 +454,7 @@ static void scan_preset_banks(const std::string dir_path, bool read_only)
 
 	for (std::vector<std::string>::iterator it = filenames.begin(); it != filenames.end(); ++it)
 		scan_preset_bank(dir_path, *it, read_only);
+#endif
 }
 
 static std::string sFactoryBanksDirectory;
