@@ -1,7 +1,7 @@
 /*
  *  Synthesizer.cpp
  *
- *  Copyright (c) 2014 Nick Dowell
+ *  Copyright (c) 2014-2016 Nick Dowell
  *
  *  This file is part of amsynth.
  *
@@ -46,6 +46,10 @@ Synthesizer::Synthesizer()
 	_voiceAllocationUnit->SetMaxVoices(config.polyphony);
 	_voiceAllocationUnit->setPitchBendRangeSemitones(config.pitch_bend_range);
 	
+	if (config.current_tuning_file != "default")
+		_voiceAllocationUnit->loadScale(config.current_tuning_file.c_str());
+
+	Preset::setIgnoredParameterNames(config.ignored_parameters);
 	_presetController = new PresetController;
 	_presetController->loadPresets(config.current_bank_file.c_str());
 	_presetController->selectPreset(0);
@@ -77,13 +81,45 @@ void Synthesizer::saveBank(const char *filename)
 
 void Synthesizer::loadState(char *buffer)
 {
-	_presetController->getCurrentPreset().fromString(buffer);
+	if (!_presetController->getCurrentPreset().fromString(buffer))
+		return;
+
+	std::istringstream input (buffer);
+	for (std::string line; std::getline(input, line); ) {
+		std::istringstream stream (line);
+		std::string type, key, value;
+		stream >> type;
+
+		if (type == "<property>") {
+			stream >> key;
+			stream.get(); // skip whitespace
+			std::getline(stream, value); // value may contain whitespace
+
+			if (key == PROP_KBM_FILE)
+				loadTuningKeymap(value.c_str());
+
+			if (key == PROP_SCL_FILE)
+				loadTuningScale(value.c_str());
+		}
+	}
 }
 
 int Synthesizer::saveState(char **buffer)
 {
-	std::string string = _presetController->getCurrentPreset().toString();
-	return asprintf(buffer, "%s", string.c_str());
+	std::stringstream stream;
+	_presetController->getCurrentPreset().toString(stream);
+
+	const std::string &tuning_kbm_file = _voiceAllocationUnit->tuningMap.getKeyMapFile();
+	if (tuning_kbm_file.length())
+		stream << "<property> " PROP_KBM_FILE " " << tuning_kbm_file << std::endl;
+
+	const std::string &tuning_scl_file = _voiceAllocationUnit->tuningMap.getScaleFile();
+	if (tuning_scl_file.length())
+		stream << "<property> " PROP_SCL_FILE " " << tuning_scl_file << std::endl;
+
+	std::string string = stream.str();
+	*buffer = (char *)malloc(4096);
+	return sprintf(*buffer, "%s", string.c_str());
 }
 
 const char *Synthesizer::getPresetName(int presetNumber)
@@ -158,17 +194,20 @@ void Synthesizer::setMaxNumVoices(int value)
 
 int Synthesizer::loadTuningKeymap(const char *filename)
 {
-	return _voiceAllocationUnit->loadKeyMap(filename);
+	if (filename && strlen(filename))
+		return _voiceAllocationUnit->loadKeyMap(filename);
+
+	_voiceAllocationUnit->tuningMap.defaultKeyMap();
+	return 0;
 }
 
 int Synthesizer::loadTuningScale(const char *filename)
 {
-	return _voiceAllocationUnit->loadScale(filename);
-}
+	if (filename && strlen(filename))
+		return _voiceAllocationUnit->loadScale(filename);
 
-void Synthesizer::defaultTuning()
-{
-	_voiceAllocationUnit->defaultTuning();
+	_voiceAllocationUnit->tuningMap.defaultScale();
+	return 0;
 }
 
 void Synthesizer::setSampleRate(int sampleRate)
@@ -177,15 +216,15 @@ void Synthesizer::setSampleRate(int sampleRate)
 	_voiceAllocationUnit->SetSampleRate(sampleRate);
 }
 
-static bool comapare(const amsynth_midi_event_t &first, const amsynth_midi_event_t &second) { return (first.offset_frames < second.offset_frames); }
-
-void Synthesizer::process(unsigned int nframes, std::vector<amsynth_midi_event_t> &midi_in, float *audio_l, float *audio_r, unsigned audio_stride)
+void Synthesizer::process(unsigned int nframes,
+						  const std::vector<amsynth_midi_event_t> &midi_in,
+						  std::vector<amsynth_midi_cc_t> &midi_out,
+						  float *audio_l, float *audio_r, unsigned audio_stride)
 {
 	if (_sampleRate < 0) {
 		assert(!"sample rate has not been set");
 		return;
 	}
-	std::sort(midi_in.begin(), midi_in.end(), comapare);
 	std::vector<amsynth_midi_event_t>::const_iterator event = midi_in.begin();
 	unsigned frames_left_in_buffer = nframes, frame_index = 0;
 	while (frames_left_in_buffer) {
@@ -211,4 +250,5 @@ void Synthesizer::process(unsigned int nframes, std::vector<amsynth_midi_event_t
 		_midiController->HandleMidiData(event->buffer, event->length);
 		++event;
 	}
+	_midiController->generateMidiOutput(midi_out);
 }
