@@ -1,7 +1,7 @@
 /*
  *  Synthesizer.cpp
  *
- *  Copyright (c) 2014 Nick Dowell
+ *  Copyright (c) 2014-2017 Nick Dowell
  *
  *  This file is part of amsynth.
  *
@@ -27,11 +27,24 @@
 
 #include <vector>
 
+//
+// Property names
+//
+#define PROP_KBM_FILE "tuning_kbm_file"
+#define PROP_SCL_FILE "tuning_scl_file"
+
+
 class MidiController;
 class PresetController;
 class VoiceAllocationUnit;
 
-class Synthesizer
+struct ISynthesizer
+{
+    virtual int loadTuningKeymap(const char *filename) = 0;
+    virtual int loadTuningScale(const char *filename) = 0;
+};
+
+class Synthesizer : ISynthesizer
 {
 public:
     
@@ -65,21 +78,21 @@ public:
 	int getMaxNumVoices();
 	void setMaxNumVoices(int value);
 
-	int loadTuningKeymap(const char *filename);
-	int loadTuningScale(const char *filename);
-	void defaultTuning();
+	virtual int loadTuningKeymap(const char *filename);
+	virtual int loadTuningScale(const char *filename);
 
 	void setSampleRate(int sampleRate);
 
-    void process(unsigned nframes,
-                 std::vector<amsynth_midi_event_t> &midi_in,
-                 float *audio_l, float *audio_r,
-                 unsigned audio_stride = 1);
-    
+	void process(unsigned nframes,
+				 const std::vector<amsynth_midi_event_t> &midi_in,
+				 std::vector<amsynth_midi_cc_t> &midi_out,
+				 float *audio_l, float *audio_r, unsigned audio_stride = 1);
+
     MidiController *getMidiController() DEPRECATED { return _midiController; };
     PresetController *getPresetController() DEPRECATED { return _presetController; }
+    VoiceAllocationUnit *getVoiceAllocationUnit() DEPRECATED { return _voiceAllocationUnit; }
     
-private:
+// private:
 
     double _sampleRate;
     MidiController *_midiController;
