@@ -38,7 +38,7 @@ class ALSAMidiDriver : public MidiDriver
 public:
 	ALSAMidiDriver(const char *client_name);
 	virtual ~ALSAMidiDriver		( );
-    virtual int read(unsigned char *bytes, unsigned maxBytes);
+    virtual int read(unsigned char *buffer, unsigned maxBytes);
     virtual int write_cc(unsigned int channel, unsigned int param, unsigned int value);
     virtual int open();
     virtual int close();
@@ -52,19 +52,22 @@ private:
 };
 
 int
-ALSAMidiDriver::read(unsigned char *bytes, unsigned maxBytes)
+ALSAMidiDriver::read(unsigned char *buffer, unsigned maxBytes)
 {
 	if (seq_handle == NULL) {
 		return 0;
 	}
-	snd_seq_event_t *ev = NULL;
-	int res = snd_seq_event_input( seq_handle, &ev );
-	if (res == -EAGAIN) {
-		return 0;
+	unsigned char *ptr = buffer;
+	while (1) {
+		snd_seq_event_t *ev = NULL;
+		int res = snd_seq_event_input(seq_handle, &ev);
+		if (res < 0)
+			break;
+		ptr += snd_midi_event_decode(seq_midi_parser, ptr, maxBytes - (ptr - buffer), ev);
+		if (res < 1)
+			break;
 	}
-	int num_bytes = snd_midi_event_decode( seq_midi_parser, bytes, maxBytes, ev );
-	snd_seq_free_event( ev );
-	return num_bytes;
+	return (int)(ptr - buffer);
 }
 
 int
@@ -90,8 +93,6 @@ ALSAMidiDriver::write_cc(unsigned int channel, unsigned int param, unsigned int
       cout << "param = " << param << " value = " << value << " ret = " << ret << endl;
 #endif
       if (ret < 0 ) cout << snd_strerror(ret) << endl;        
-      snd_seq_free_event( &ev );
-
       return ret;
 }
 
@@ -140,7 +141,11 @@ int ALSAMidiDriver::open()
 	
 	snd_seq_poll_descriptors( seq_handle, &pollfd_in, 1, POLLIN );
 
-	config.current_midi_driver = "ALSA";
+	if (config.current_midi_driver.empty()) {
+		config.current_midi_driver = "ALSA";
+	} else {
+		config.current_midi_driver += " + ALSA";
+	}
 	config.alsa_seq_client_id = snd_seq_client_id(seq_handle);
 
 	return 0;
@@ -152,7 +157,7 @@ ALSAMidiDriver::ALSAMidiDriver(const char *client_name)
 	seq_handle = NULL;
 	memset( &pollfd_in, 0, sizeof(pollfd_in) );
 	if( snd_midi_event_new( 32, &seq_midi_parser ) )
-		cout << "Error creating midi event parser\n";
+		cout << "Error creating MIDI event parser\n";
 }
 
 ALSAMidiDriver::~ALSAMidiDriver()
