@@ -1,7 +1,7 @@
 /*
  *  main.cc
  *
- *  Copyright (c) 2001-2012 Nick Dowell
+ *  Copyright (c) 2001-2016 Nick Dowell
  *
  *  This file is part of amsynth.
  *
@@ -50,12 +50,17 @@
 #include <getopt.h>
 #include <unistd.h>
 #include <string>
+#include <climits>
 #include <cstring>
 #include <stdlib.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <algorithm>
+
+#include "gettext.h"
+#define _(string) gettext (string)
 
 using namespace std;
 
@@ -66,31 +71,6 @@ using namespace std;
 #endif
 
 
-
-string help_text =
-"usage: " PACKAGE " [options]\n\
-\n\
-Any options given here override those in the config file ($HOME/.amSynthrc)\n\
-\n\
-OPTIONS:\n\
-\n\
-	-h          show this usage message\n\
-	-v          show version information\n\
-	-d          show some debugging output\n\
-\n\
-	-b <filename>   use <filename> as the bank to store presets\n\
-	-t <filename>   use <filename> as a tuning file\n\
-\n\
-	-a <string> set the sound output driver to use [alsa/oss/auto(default)]\n\
-	-r <int>    set the sampling rate to use\n\
-	-m <string>	set the midi driver to use [alsa/oss/auto(default)]\n\
-	-c <int>    set the midi channel to respond to (default=all)\n\
-	-p <int>    set the polyphony (maximum active voices)\n\
-\n\
-	-n <name>   specify the JACK client name to use\n\
-	--jack_autoconnect=<true|false>\n\
-\n";
-
 Configuration & config = Configuration::get();
 
 #ifdef ENABLE_REALTIME
@@ -103,7 +83,7 @@ void sched_realtime()
 	sched_getparam(0, &sched);
 
 	if (foo) {
-		DEBUGMSG("Failed to set SCHED_FIFO\n");
+		DEBUGMSG(_("Failed to set SCHED_FIFO\n"));
 		config.realtime = 0;
 	}
 	else {
@@ -122,12 +102,12 @@ int fcopy (const char * dest, const char *source)
 {
 	FILE *in = fopen (source,"r");
 	if (in == NULL) {
-		fprintf (stderr, "error reading source file %s\n", source);
+		fprintf (stderr, _("error reading source file %s\n"), source);
 		return -1;
 	}
 	FILE *out = fopen (dest,"w");
 	if (out == NULL) {
-		fprintf (stderr, "error creating destination file %s\n", dest);
+		fprintf (stderr, _("error creating destination file %s\n"), dest);
 		return -1;
 	}
 	fseek (in, 0, SEEK_END);
@@ -162,12 +142,12 @@ void install_default_files_if_reqd()
 	
 	if (stat (user_config, &st) == -1)
 	{
-		printf ("installing configuration file to %s\n", user_config);
+		printf (_("installing configuration file to %s\n"), user_config);
 		fcopy (user_config, factory_config);
 	}
 	if (stat (user_bank, &st) == -1)
 	{
-		printf ("installing default sound bank to %s\n", user_bank);
+		printf (_("installing default sound bank to %s\n"), user_bank);
 		fcopy (user_bank, factory_bank);
 	}
 
@@ -184,7 +164,6 @@ void ptest ();
 
 static MidiDriver *midiDriver;
 Synthesizer *s_synthesizer;
-static void amsynth_audio_callback(float *buffer_l, float *buffer_r, unsigned num_frames, int stride, amsynth_midi_event_t *events, unsigned event_count);
 static unsigned char *midiBuffer;
 static const size_t midiBufferSize = 4096;
 static int gui_midi_pipe[2];
@@ -228,7 +207,7 @@ GenericOutput * open_audio()
 			
 			// we were asked specifically for jack, so don't use anything else
 			if (config.audio_driver == "jack") {
-				std::cerr << "JACK init failed: " << jack_error << "\n";
+				std::cerr << _("JACK init failed: ") << jack_error << "\n";
 				return new NullAudioOutput;
 			}
 		}
@@ -254,23 +233,23 @@ static void open_midi()
 	
 	if (config.midi_driver == "alsa" || config.midi_driver == "ALSA") {
 		if (!(midiDriver = opened_midi_driver(CreateAlsaMidiDriver(alsa_client_name)))) {
-			std::cerr << "error: could not open ALSA MIDI interface";
+			std::cerr << _("error: could not open ALSA MIDI interface") << endl;
 		}
 		return;
 	}
 
 	if (config.midi_driver == "oss" || config.midi_driver == "OSS") {
 		if (!(midiDriver = opened_midi_driver(CreateOSSMidiDriver()))) {
-			std::cerr << "error: could not open OSS MIDI interface";
+			std::cerr << _("error: could not open OSS MIDI interface") << endl;
 		}
 		return;
 	}
 
 	if (config.midi_driver == "auto") {
-		if (!(midiDriver = opened_midi_driver(CreateAlsaMidiDriver(alsa_client_name)))) {
-			if (!(midiDriver = opened_midi_driver(CreateOSSMidiDriver()))) {
-				std::cerr << "error: could not open any MIDI interface";
-			}
+		midiDriver = opened_midi_driver(CreateAlsaMidiDriver(alsa_client_name)) ?:
+		             opened_midi_driver(CreateOSSMidiDriver());
+		if (config.current_midi_driver.empty()) {
+			std::cerr << _("error: could not open any MIDI interface") << endl;
 		}
 		return;
 	}
@@ -296,31 +275,31 @@ static void signal_handler(int signal)
 
 int main( int argc, char *argv[] )
 {
+	srand(time(NULL));
+
 #ifdef ENABLE_REALTIME
 	sched_realtime();
-#endif
 
-	srand(time(NULL));
-	
 	// need to drop our suid-root permissions :-
 	// GTK will not work SUID for security reasons..
-	setreuid( getuid(), getuid() );
-	setregid( getgid(), getgid() );	
-
-#ifdef WITH_GUI
-	bool no_gui = (getenv("AMSYNTH_NO_GUI") != NULL);
-
-	if (!no_gui)
-		gui_kit_init(argc, argv);
-#else
-  bool no_gui = true;
+	setreuid(getuid(), getuid());
+	setregid(getgid(), getgid());
 #endif
-	
+
+#ifdef ENABLE_NLS
+	setlocale(LC_ALL, "");
+	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALEDIR);
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+	textdomain(GETTEXT_PACKAGE);
+#endif
+
 	int initial_preset_no = 0;
 
 	// needs to be called before our own command line parsing code
 	amsynth_lash_process_args(&argc, &argv);
 	
+	bool no_gui = (getenv("AMSYNTH_NO_GUI") != NULL);
+
 	static struct option longopts[] = {
 		{ "jack_autoconnect", optional_argument, NULL, 0 },
 		{ 0 }
@@ -333,7 +312,30 @@ int main( int argc, char *argv[] )
                 cout << PACKAGE_STRING << endl;
 				return 0;
 			case 'h':
-				cout << help_text;
+				cout << _("usage: ") << PACKAGE << _(" [options]") << endl
+				     << endl
+				     << _("Any options given here override those in the config file ($HOME/.amSynthrc)") << endl
+				     << endl
+				     << _("OPTIONS:") << endl
+				     << endl
+				     << _("	-h          show this usage message") << endl
+				     << _("	-v          show version information") << endl
+				     << _("	-x          run in headless mode (without GUI)") << endl
+				     << endl
+				     << _("	-b <file>   use <file> as the bank to store presets") << endl
+				     << _("	-t <file>   use <file> as a tuning file") << endl
+				     << endl
+				     << _("	-a <string> set the sound output driver to use [alsa/oss/auto(default)]") << endl
+				     << _("	-r <int>    set the sampling rate to use") << endl
+				     << _("	-m <string> set the MIDI driver to use [alsa/oss/auto(default)]") << endl
+				     << _("	-c <int>    set the MIDI channel to respond to (default=all)") << endl
+				     << _("	-p <int>    set the polyphony (maximum active voices)") << endl
+				     << endl
+				     << _("	-n <name>   specify the JACK client name to use") << endl
+				     << _("	--jack_autoconnect[=<true|false>]") << endl
+				     << _("	            automatically connect jack audio ports to hardware I/O ports. (Default: true)") << endl
+				     << endl;
+
 				return 0;
 			case 'z':
 				ptest();
@@ -356,9 +358,6 @@ int main( int argc, char *argv[] )
 			case 'a':
 				config.audio_driver = optarg;
 				break;
-			case 'd':
-				config.debug_drivers = 1;
-				break;
 			case 'r':
 				config.sample_rate = atoi( optarg );
 				break;
@@ -384,32 +383,36 @@ int main( int argc, char *argv[] )
 		}
 	}
 
-	// all config files should eventually be migrated to the ~./amsynth directory
-	mkdir ((std::string(getenv("HOME")) + std::string("/.amsynth")).c_str(), 0000755);
-	mkdir ((std::string(getenv("HOME")) + std::string("/.amsynth") + std::string("/banks")).c_str(), 0000755);
+#ifdef WITH_GUI
+	if (!no_gui)
+		gui_kit_init(&argc, &argv);
+#endif
+
+	/* all config files should eventually be migrated to the ~./amsynth directory */ {
+		char *path = NULL;
+		if (asprintf(&path, "%s/.amsynth", getenv("HOME")) > 0) {
+			mkdir(path, 0000755);
+			free(path), path = NULL;
+		}
+		if (asprintf(&path, "%s/.amsynth/banks", getenv("HOME")) > 0) {
+			mkdir(path, 0000755);
+			free(path), path = NULL;
+		}
+	}
 
 	install_default_files_if_reqd();
 
-	
-	if (config.debug_drivers)
-		cout << "\n*** CONFIGURATION:\n"
-				<< "MIDI:- driver:" << config.midi_driver 
-				<< " channel:" << config.midi_channel << endl 
-				<< "AUDIO:- driver:" << config.audio_driver 
-				<< " sample rate:" << config.sample_rate << endl;
-
 	string amsynth_bank_file = config.current_bank_file;
 	// string amsynth_tuning_file = config.current_tuning_file;
 
 	GenericOutput *out = open_audio();
 	if (!out)
-		fatal_error("Fatal Error: open_audio() returned NULL.\n"
+		fatal_error(std::string(_("Fatal Error: open_audio() returned NULL.\n")) +
 					"config.audio_driver = " + config.audio_driver);
 
 	// errors now detected & reported in the GUI
 	out->init();
-	out->setAudioCallback(&amsynth_audio_callback);
-	
+
 	s_synthesizer = new Synthesizer();
 	s_synthesizer->setSampleRate(config.sample_rate);
 	
@@ -424,7 +427,6 @@ int main( int argc, char *argv[] )
 	
 	open_midi();
 	midiBuffer = (unsigned char *)malloc(midiBufferSize);
-    s_synthesizer->getMidiController()->setMidiDriver(midiDriver);
 
 	// prevent lash from spawning a new jack server
 	setenv("JACK_NO_START_SERVER", "1", 0);
@@ -453,18 +455,19 @@ int main( int argc, char *argv[] )
 		gui_dealloc();
 	} else {
 #endif
-		printf("amsynth running in headless mode, press ctrl-c to exit\n");
+		printf(_("amsynth running in headless mode, press ctrl-c to exit\n"));
 		signal(SIGINT, &signal_handler);
 		while (!signal_received)
 			sleep(2); // delivery of a signal will wake us early
-		printf("shutting down...\n");
+		printf("\n");
+		printf(_("shutting down...\n"));
 #ifdef WITH_GUI
 	}
 #endif
 
 	out->Stop ();
 
-	if (config.xruns) std::cerr << config.xruns << " audio buffer underruns occurred\n";
+	if (config.xruns) std::cerr << config.xruns << _(" audio buffer underruns occurred\n");
 
 	delete out;
 	return 0;
@@ -474,20 +477,28 @@ unsigned
 amsynth_timer_callback()
 {
 	amsynth_lash_poll_events();
-    s_synthesizer->getMidiController()->timer_callback();
 	return 1;
 }
 
 void amsynth_midi_input(unsigned char status, unsigned char data1, unsigned char data2)
 {
 	unsigned char buffer[3] = { status, data1, data2 };
+	if (config.midi_channel > 1) {
+		buffer[0] |= ((config.midi_channel - 1) & 0x0f);
+	}
 	write(gui_midi_pipe[1], buffer, sizeof(buffer));
 }
 
-void
-amsynth_audio_callback(float *buffer_l, float *buffer_r, unsigned num_frames, int stride, amsynth_midi_event_t *events, unsigned event_count)
+static bool compare(const amsynth_midi_event_t &first, const amsynth_midi_event_t &second) {
+	return (first.offset_frames < second.offset_frames);
+}
+
+void amsynth_audio_callback(
+		float *buffer_l, float *buffer_r, unsigned num_frames, int stride,
+		const std::vector<amsynth_midi_event_t> &midi_in,
+		std::vector<amsynth_midi_cc_t> &midi_out)
 {
-	std::vector<amsynth_midi_event_t> midi_in(events, events + event_count);
+	std::vector<amsynth_midi_event_t> midi_in_merged = midi_in;
 
 	if (midiBuffer) {
 		unsigned char *buffer = midiBuffer;
@@ -501,7 +512,7 @@ amsynth_audio_callback(float *buffer_l, float *buffer_r, unsigned num_frames, in
 				event.offset_frames = num_frames - 1;
 				event.length = bytes_read;
 				event.buffer = buffer;
-				midi_in.push_back(event);
+				midi_in_merged.push_back(event);
 				buffer += bytes_read;
 				bufferSize -= bytes_read;
 			}
@@ -514,13 +525,22 @@ amsynth_audio_callback(float *buffer_l, float *buffer_r, unsigned num_frames, in
 				event.offset_frames = num_frames - 1;
 				event.length = bytes_read;
 				event.buffer = buffer;
-				midi_in.push_back(event);
+				midi_in_merged.push_back(event);
 			}
 		}
 	}
 
+	std::sort(midi_in_merged.begin(), midi_in_merged.end(), compare);
+
 	if (s_synthesizer) {
-		s_synthesizer->process(num_frames, midi_in, buffer_l, buffer_r, stride);
+		s_synthesizer->process(num_frames, midi_in_merged, midi_out, buffer_l, buffer_r, stride);
+	}
+
+	if (midiDriver && !midi_out.empty()) {
+		std::vector<amsynth_midi_cc_t>::const_iterator out_it;
+		for (out_it = midi_out.begin(); out_it != midi_out.end(); ++out_it) {
+			midiDriver->write_cc(out_it->channel, out_it->cc, out_it->value);
+		}
 	}
 }
 
@@ -541,7 +561,7 @@ amsynth_load_tuning_file(const char *filename)
 {
 	int result = s_synthesizer->loadTuningScale(filename);
 	if (result != 0) {
-		cerr << "error: could not load tuning file " << filename << endl;
+		cerr << _("error: could not load tuning file ") << filename << endl;
 	}
 	return result;
 }
@@ -603,8 +623,8 @@ void ptest ()
 	unsigned long usec_audio = kTimeSeconds * kNumVoices * 1000000;
 	unsigned long usec_cpu = user_usec + syst_usec;
 	
-	fprintf (stderr, "user time: %f		system time: %f\n", user_usec/1000000.f, syst_usec/1000000.f);
-	fprintf (stderr, "performance index: %f\n", (float) usec_audio / (float) usec_cpu);
+	fprintf (stderr, _("user time: %f		system time: %f\n"), user_usec/1000000.f, syst_usec/1000000.f);
+	fprintf (stderr, _("performance index: %f\n"), (float) usec_audio / (float) usec_cpu);
 	
 	delete [] buffer;
 	delete voiceAllocationUnit;
